Extracts comments from Elixir source while preserving source positions.
Summary
Functions
@spec extract(String.t()) :: [ExAST.Comment.t()]
Extracts comments from Elixir source while preserving source positions.
@spec extract(String.t()) :: [ExAST.Comment.t()]